/* Copyright (c) 2023 Skyward Experimental Rocketry
 * Author: Alberto Nidasio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#pragma once

#include "interfaces/gpio.h"

#define MIOSIX_SENSOR_H3LIS331DL_SPI SPI1
#define MIOSIX_SENSOR_LPS22DF_SPI SPI1
#define MIOSIX_SENSOR_LSM6DSRX_SPI SPI3
#define MIOSIX_SENSOR_LIS2MDL_SPI SPI3
#define MIOSIX_SENSOR_UBXGPS_SPI SPI3
#define MIOSIX_SENSOR_ADS131M08_SPI SPI4

#define MIOSIX_SENSOR_LPS28DFW_I2C I2C1

#define MIOSIX_SENSOR_VN100_SYNC_IN EXTI7_IRQHandlerImpl
#define MIOSIX_SENSOR_VN100_SPI SPI1

#define MIOSIX_RADIO_DIO0_IRQ EXTI11_IRQHandlerImpl
#define MIOSIX_RADIO_DIO1_IRQ EXTI3_IRQHandlerImpl
#define MIOSIX_RADIO_DIO3_IRQ EXTI6_IRQHandlerImpl
#define MIOSIX_RADIO_SPI SPI6

#define MIOSIX_SERVOS_1_TIM TIM4
#define MIOSIX_SERVOS_1_CHANNEL CHANNEL_2
#define MIOSIX_SERVOS_2_TIM TIM3
#define MIOSIX_SERVOS_2_CHANNEL CHANNEL_1
#define MIOSIX_SERVOS_3_TIM TIM9
#define MIOSIX_SERVOS_3_CHANNEL CHANNEL_1
#define MIOSIX_BUZZER_TIM TIM1
#define MIOSIX_BUZZER_CHANNEL CHANNEL_1

#define MIOSIX_PARAFOIL_SERVO_1_TIM MIOSIX_SERVOS_1_TIM
#define MIOSIX_PARAFOIL_SERVO_1_CHANNEL MIOSIX_SERVOS_1_CHANNEL
#define MIOSIX_PARAFOIL_SERVO_2_TIM MIOSIX_SERVOS_2_TIM
#define MIOSIX_PARAFOIL_SERVO_2_CHANNEL MIOSIX_SERVOS_2_CHANNEL
#define MIOSIX_AIRBRAKES_TIM MIOSIX_SERVOS_2_TIM
#define MIOSIX_AIRBRAKES_CHANNEL MIOSIX_SERVOS_2_CHANNEL
#define MIOSIX_EXPULSION_TIM MIOSIX_SERVOS_3_TIM
#define MIOSIX_EXPULSION_CHANNEL MIOSIX_SERVOS_3_CHANNEL


namespace miosix
{

namespace interfaces
{

namespace spi1
{
using sck  = Gpio<GPIOA_BASE, 5>;
using miso = Gpio<GPIOA_BASE, 6>;
using mosi = Gpio<GPIOA_BASE, 7>;
}  // namespace spi1

namespace spi3
{
using sck  = Gpio<GPIOB_BASE, 3>;
using miso = Gpio<GPIOB_BASE, 4>;
using mosi = Gpio<GPIOD_BASE, 6>;
}  // namespace spi3

namespace spi4
{
using sck  = Gpio<GPIOE_BASE, 2>;
using miso = Gpio<GPIOE_BASE, 5>;
using mosi = Gpio<GPIOE_BASE, 6>;
}  // namespace spi4

namespace i2c1
{
using sda = Gpio<GPIOB_BASE, 9>;
using scl = Gpio<GPIOB_BASE, 8>;
}  // namespace i2c1

namespace usart1
{
using tx = Gpio<GPIOA_BASE, 9>;
using rx = Gpio<GPIOA_BASE, 10>;
}  // namespace usart1

namespace usart2
{
using tx = Gpio<GPIOA_BASE, 2>;
using rx = Gpio<GPIOA_BASE, 3>;
}  // namespace usart2

namespace can2
{
using tx = Gpio<GPIOB_BASE, 12>;
using rx = Gpio<GPIOB_BASE, 13>;
}  // namespace can2

}  // namespace interfaces

namespace peripherals
{

namespace leds
{
using userLed1   = Gpio<GPIOB_BASE, 7>;
using userLed2   = Gpio<GPIOE_BASE, 3>;
using userLed3_1 = Gpio<GPIOC_BASE, 13>;  // On MCU rev 2
using userLed3_2 = Gpio<GPIOG_BASE, 9>;   // On MCU rev 3
using userLed4   = Gpio<GPIOC_BASE, 2>;
}  // namespace leds

namespace switches
{
using userSwitch1 = Gpio<GPIOB_BASE, 2>;
}

namespace servos
{
using servo1 = Gpio<GPIOC_BASE, 6>;  // TIM8 CH1
using servo2 = Gpio<GPIOC_BASE, 7>;  // TIM8 CH2
}  // namespace servos

}  // namespace peripherals

namespace sensors
{
namespace LSM6DSRX
{
using cs   = Gpio<GPIOC_BASE, 4>;
using int1 = Gpio<GPIOD_BASE, 13>;
using int2 = Gpio<GPIOG_BASE, 7>;
}  // namespace LSM6DSRX

namespace H3LIS331DL
{
using cs   = Gpio<GPIOD_BASE, 3>;
using int1 = Gpio<GPIOC_BASE, 3>;
}  // namespace H3LIS331DL

namespace LIS2MDL
{
using cs = Gpio<GPIOD_BASE, 5>;
}  // namespace LIS2MDL

namespace LPS22DF
{
using cs   = Gpio<GPIOD_BASE, 7>;
using int1 = Gpio<GPIOB_BASE, 11>;
}  // namespace LPS22DF

namespace ADS131M08
{
using cs = Gpio<GPIOG_BASE, 10>;
}  // namespace ADS131M08

namespace UBXGps
{
using cs = Gpio<GPIOG_BASE, 7>;
}

namespace VN100
{
using cs      = Gpio<GPIOA_BASE, 15>;
using syncIn  = Gpio<GPIOB_BASE, 7>;
using syncOut = Gpio<GPIOE_BASE, 3>;
}  // namespace VN100

namespace tank_level
{
using lvl1 = Gpio<GPIOB_BASE, 1>;
using lvl2 = Gpio<GPIOG_BASE, 6>;
using lvl3 = Gpio<GPIOB_BASE, 14>;
}  // namespace tank_level

namespace battery_voltage
{
using ch15 = Gpio<GPIOC_BASE, 5>;
}

}  // namespace sensors


namespace radio
{
using cs   = Gpio<GPIOC_BASE, 4>;
using txEn = Gpio<GPIOC_BASE, 5>;
using rxEn = Gpio<GPIOB_BASE, 15>;
using dio0 = Gpio<GPIOD_BASE, 11>;
using dio1 = Gpio<GPIOG_BASE, 3>;
using dio3 = Gpio<GPIOG_BASE, 6>;
using nrst = Gpio<GPIOE_BASE, 4>;
}  // namespace radio

namespace sense
{
using detachPayload  = Gpio<GPIOC_BASE, 3>;
using detachMain     = Gpio<GPIOA_BASE, 4>;
using detachRamp     = Gpio<GPIOB_BASE, 11>;
using expulsionSense = Gpio<GPIOA_BASE, 3>;
using cutterSense    = Gpio<GPIOC_BASE, 1>;
}  // namespace sense

namespace gpios
{
using mainDeploy = Gpio<GPIOD_BASE, 4>;
}  // namespace gpios

}  // namespace miosix
